#pragma once

#ifndef __D12GRAPH__
#define __D12GRAPH__

#include <Windows.h>
#include <VersionHelpers.h>

#include <d2d1_3.h>
#include <dwrite.h>
#include <d3d11on12.h>
#include <d3d12.h>
#include <dxgi1_6.h>
#include <D3Dcompiler.h>
#include <DirectXMath.h>

#include "d3dx12.h"

#include <wrl.h>

#include <Wincodec.h>

#include <vector>

#include "Config.h"

const short GPU_PRIMARY{ 0 };

struct D12Interface {
	Microsoft::WRL::ComPtr<IDXGIAdapter1> Adapter{ nullptr };
	bool Hardware{};
	std::wstring GPU_Name{};
	D3D_FEATURE_LEVEL D3D_Level{};
	std::wstring strLevel{};
};

struct Screen {
	int Width{};
	int Height{};
	DXGI_OUTPUT_DESC desc{};
	int DPI{};
	int OutputNum{};
	int ModeNum{};
};

using namespace Microsoft::WRL;

class D12Graphics {
private:
	std::vector<D12Interface> InterfaceList;
	short GPU_Num{ GPU_PRIMARY };

	std::vector<Screen> CurrentOutput{};

public:
	D12Graphics();
	~D12Graphics();

	// Check and List availale hardware

	bool CheckVersionSupport();
	bool CheckSupport();

	bool GetResolutions(Config &);

	void AdapterList(D12Interface&);

	inline size_t GetInterfacesNum() { return InterfaceList.size(); }

	inline void SetInterface(short number) { GPU_Num = number; }

	// Init D3D

	bool InitGraphics(HWND&);
	void Render();
	void Destroy();

	inline void SetBgkColor(float r, float g, float b, float a) {
		clearColor[0] = r;
		clearColor[1] = g;
		clearColor[2] = b;
		clearColor[3] = a;
	}

private:

	IDXGIFactory4 * factory{ nullptr };
	ID3D12Device * pDevice{ nullptr };
	ComPtr<IDXGISwapChain3> pSwapChain{ nullptr };

	HWND MainWnd{};

	float Width{};
	float Height{};
	static const UINT FrameCount = 3;

	float AspectRatio{};	
	UINT FrameIndex{};

	D3D12_VIEWPORT Viewport{};
	D3D12_RECT ScissorRect{};

	bool CreateSwapChains();
	bool CreateDescriptors();
	void SetAspectRatio(float p) { AspectRatio = p; }

	void PopulateCommandList();

	void MoveToNextFrame();

	ComPtr<ID3D12CommandAllocator> pCommandAllocators[FrameCount]{ nullptr };
	ComPtr<ID3D12CommandQueue> pCommandQueue{ nullptr };
	ComPtr<ID3D12Resource> pRenderTargets[FrameCount]{ nullptr };
	ComPtr<ID3D12RootSignature> pRootSignature{ nullptr };
	ComPtr<ID3D12DescriptorHeap> pRtvHeap{ nullptr };

	ComPtr<ID3D12GraphicsCommandList> pCommandList{};
	std::vector<ID3D12CommandList*> ppCommandLists{};

	HANDLE hFenceEvent{};
	ComPtr<ID3D12Fence> pFence{ nullptr };
	UINT64 FenceValues[FrameCount];

	UINT RtvDescriptorSize{};

	float clearColor[4] = { 0.2f, 0.2f, 0.2f, 0.0f };
};

#endif __D12GRAPH__