#include "Graphics.h"

#include "WinMain.h"

#include "Logger.h"
#include "Config.h"

#include <Windows.h>

#include <memory>

class Entry : public WMainProg {
public:
	std::unique_ptr<D12Graphics> Gfx;
public:
	Entry() { Log_Open("Logger.txt"); Log_Write("Start"); };
	~Entry() { Gfx->Destroy();  Log_Write("Close"); Log_Close(); };

	void wndproc(MSG uMsg) { }

	void Loop();
	void Start();
};

void Entry::Start() {
	SetWindowName(L"DirectX12 - Plain Window");
	Gfx->InitGraphics(MainWnd);
	Gfx->SetBgkColor(0.5f, 0.8f, 0.0f, 1.0f);
}

void Entry::Loop() {
	Gfx->Render();
}

int WINAPI wWinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPWSTR, int) {
	auto main = std::make_unique <Entry>();
	main->Gfx = std::make_unique<D12Graphics>();

	if (!main->Gfx->CheckSupport()) {
		std::cout << "No Support for DirectX12\n";
		Log_Write("No Support for DirectX12\n");

		return 1;
	}

	main->Gfx->CheckVersionSupport();

	Config cfg{};

	LoadConfig(cfg, L"Config.cfg");
	if (!main->Gfx->GetResolutions(cfg)) {
		Log_Write("Screen Res failed\n");

		return 1;
	}

	SaveConfig(cfg, L"Config.cfg");

	WMainClass wmc{};

	wmc.Width = cfg.Width;
	wmc.Height = cfg.Height;

	main->Create(wmc);

	return 0;
}