#include "WinMain.h"

#include "Logger.h"

#include <iostream>
#include <string>
#include <tchar.h>


LRESULT CALLBACK MainWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	switch (uMsg) {
	case WM_CREATE:

		return 0;

	case WM_QUIT:
	case WM_DESTROY:
	case WM_CLOSE:
		PostQuitMessage(0);

		return 0;
	}

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

WMainProg::WMainProg() {
}

WMainProg::~WMainProg() {
	DestroyWindow(MainWnd);
	hInst = nullptr;
}

void WMainProg::SetWindowName(std::wstring name) {
	windowstitle = name;
#ifdef DEBUG

	windowstitle += L"(DBG)";

#endif // DEBUG
	SetWindowText(MainWnd, windowstitle.c_str());
}

bool WMainProg::Create(WMainClass wmc) {
	WNDCLASSEX WndClassEx{};

	SetWindowName(L"DirectX12");

	std::wstring classstitle{ windowstitle };

	hInst = GetModuleHandle(nullptr);

	WndClassEx.cbSize = sizeof(WNDCLASSEX);
	WndClassEx.style = CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
	WndClassEx.lpfnWndProc = static_cast<WNDPROC> (MainWndProc);
	WndClassEx.hInstance = hInst;
	WndClassEx.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
	WndClassEx.lpszClassName = classstitle.c_str();

	if(!RegisterClassEx(&WndClassEx)) {
		Logger("RegisterClassEx");
		return false;
	}
	
	MainWnd = CreateWindowEx(NULL, classstitle.c_str(), windowstitle.c_str(), WS_OVERLAPPEDWINDOW, (GetSystemMetrics(SM_CXSCREEN) / 2) - (wmc.Width / 2), (GetSystemMetrics(SM_CYSCREEN) / 2) - (wmc.Height / 2), wmc.Width, wmc.Height, nullptr, nullptr, hInst, nullptr);

	if(!MainWnd) {
		Logger("Main Window creation failed");

		return false;
	}
	
	ShowWindow(MainWnd, SW_NORMAL);
	UpdateWindow(MainWnd);

	Start();

	Log_Write("Window Created");
	
	MainLoop();
	
	return true;
}

int WMainProg::MainLoop() {

	while (1) {
		if (PeekMessage(&uMsg, NULL, 0, 0, PM_REMOVE)) {
			TranslateMessage(&uMsg);
			DispatchMessage(&uMsg);

			if (uMsg.message == WM_QUIT) {
				break;
			}

			wndproc(uMsg);
		}

		Loop();
	}

	return static_cast<int> (uMsg.wParam);
}